package org.fcitx.fcitx5.android.data

import org.fcitx.fcitx5.android.utils.appContext

// Not thread-safe
class RecentlyUsed(
    val fileName: String,
    val capacity: Int
) : LinkedHashMap<String, String>(0, .75f, true) {

    companion object {
        const val DIR_NAME = "recently_used"
    }

    private val file = appContext.filesDir.resolve(DIR_NAME).run {
        mkdirs()
        resolve(fileName).apply { createNewFile() }
    }

    fun load() {
        val xs = file.readLines()
        xs.forEach {
            if (it.isNotBlank())
                put(it, it)
        }
    }

    fun save() {
        file.writeText(values.joinToString("\n"))
    }

    override fun removeEldestEntry(eldest: MutableMap.MutableEntry<String, String>?) =
        size > capacity

    fun insert(s: String) = put(s, s)

    fun toOrderedList() = values.toList().reversed()
}